﻿Imports ThinkGeo.MapSuite.Core
Imports System.Collections.ObjectModel

Public Class CustomAdornmentLayer
    Inherits ThinkGeo.MapSuite.Core.AdornmentLayer
    Dim x As Single
    Dim y As Single
    Dim legendItems As Dictionary(Of String, Style)

    Public Sub New(ByVal x As Single, ByVal y As Single, ByVal legendItems As Dictionary(Of String, Style))
        Me.x = x
        Me.y = y
        Me.legendItems = legendItems
    End Sub

    Protected Overrides Sub DrawCore(ByVal canvas As GeoCanvas, ByVal labelsInAllLayers As Collection(Of SimpleCandidate))
        canvas.DrawTextWithScreenCoordinate("My Legend", New GeoFont("Arial", 14), New GeoSolidBrush(GeoColor.StandardColors.Blue), x, y, DrawingLevel.LevelFour)
        y = y + 30
        For Each legendItem As KeyValuePair(Of String, Style) In legendItems
            If TypeOf legendItem.Value Is PointStyle Then
                DrawPoint(canvas, x, y, legendItem)
            ElseIf TypeOf legendItem.Value Is AreaStyle Then
                DrawRectangle(canvas, x, y, 8, legendItem)
            End If
            y = y + 15
        Next
    End Sub

    Private Sub DrawPoint(ByRef canvas As GeoCanvas, ByVal pX As Double, ByVal pY As Double, ByVal legendItem As KeyValuePair(Of String, Style))
        Dim worldPointShape As PointShape = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, pX, pY, canvas.Width, canvas.Height)
        legendItem.Value.Draw(New PointShape() {worldPointShape}, canvas, New Collection(Of SimpleCandidate), New Collection(Of SimpleCandidate))
        DrawText(canvas, pX, pY, legendItem.Key)
    End Sub

    Private Sub DrawRectangle(ByVal canvas As GeoCanvas, ByVal pX As Double, ByVal pY As Double, ByVal size As Double, ByVal legendItem As KeyValuePair(Of String, Style))
        Dim worldPointShape As PointShape = ExtentHelper.ToWorldCoordinate(canvas.CurrentWorldExtent, pX, pY, canvas.Width, canvas.Height)
        canvas.DrawArea(GetPointArrayOfARectangle(pX, pY, size), New GeoPen(GeoColor.StandardColors.Black, 1), CType(legendItem.Value, AreaStyle).FillSolidBrush, DrawingLevel.LevelFour, 0, 0, PenBrushDrawingOrder.BrushFirst)
        legendItem.Value.Draw(New PointShape() {worldPointShape}, canvas, New Collection(Of SimpleCandidate), New Collection(Of SimpleCandidate))
        DrawText(canvas, pX, pY, legendItem.Key)
    End Sub

    Private Function GetPointArrayOfARectangle(ByVal pX As Double, ByVal pY As Double, ByVal size As Double) As IEnumerable(Of ScreenPointF())
        Dim p1 As New ScreenPointF(pX - size / 2, pY - size / 2)
        Dim p2 As New ScreenPointF(pX + size / 2, pY - size / 2)
        Dim p3 As New ScreenPointF(pX + size / 2, pY + size / 2)
        Dim p4 As New ScreenPointF(pX - size / 2, pY + size / 2)

        Dim pointfs As ScreenPointF() = New ScreenPointF() {p1, p2, p3, p4}
        Dim result As New List(Of ScreenPointF())()
        result.Add(pointfs)
        Return result
    End Function

    Private Sub DrawText(ByRef canvas As GeoCanvas, ByVal pX As Double, ByRef pY As Double, ByVal pText As String)

        Dim fontStyle As DrawingFontStyles = DrawingFontStyles.Regular
        Dim Font As New GeoFont("Arial", 10, fontStyle)
        Dim text As DrawingRectangleF = canvas.MeasureText(pText, Font)

        ' add text
        pX = pX + 15 + (text.Width / 2)
        canvas.DrawTextWithScreenCoordinate(pText, Font, New GeoSolidBrush(GeoColor.StandardColors.Blue), pX, pY, DrawingLevel.LevelThree)

    End Sub
End Class
